/*
 * Copyright 2018 Christopher Cromer
 * Copyright 2018 Rodolfo Cuevas
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
 *
 * 3. Neither the name of the copyright holder nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <stdio.h>
#include <time.h>
#include "timer.h"

/**
 * Cuando empezó el timer
 */
static time_t start_time = 0;
/**
 * Cuando terminó el timer
 */
static time_t stop_time = 0;

/**
 * Empezar el timer
 */
void start_timer() {
	if (start_time != 0) {
		fprintf(stderr, "Error: El temporizador ya comenzó!\n");
	}
	else {
		time(&start_time);
		stop_time = 0;
	}
}

/**
 * Terminar el timer
 */
void stop_timer() {
	if (start_time == 0) {
		fprintf(stderr, "Error: El temporizador no ha comenzado todavía!\n");
	}
	else {
		time(&stop_time);
	}
}

/**
 * Imprimir el tiempo de ejecución
 */
void print_timer() {
	fprintf(stdout, "Tiempo de ejecución: %ld segundos\n", (long int) (stop_time - start_time));
}

/**
 * Devolver el tiempo que demoró en ejecutarse
 * @return El tiempo que demoró
 */
long int get_timer() {
	return (long int) (stop_time - start_time);
}
